# vim: sw=4
"""PlacementTool

Tool used to place objects on the canvas.

This class overrules the Placement tool that is coded in the diacanvas C code.
Since we can not instantiate Python objects from the C implementation.
"""

__author__ = 'Arjan J. Molenaar'
__revision__ = '$revision$'
__date__ = '$date$'

import diacanvas as dia
import gobject
import traceback

class PlacementTool(dia.Tool):
    __gsignals__ = {
        'button_press_event': 'override',
    	'button_release_event': 'override'
    }
    def __init__(self, type, **properties):
	self.__gobject_init__()
	self.type = type
	self.properties = properties

    def do_button_press_event(self, view, event):
	#old_allow_undo = view.canvas.get_property ('allow_undo')
	#view.canvas.set_property ('allow_undo', 0)
	
	try:
	    self.new_object = self._create_item(view, event)
	except Exception, e:
	    print 'diacanvas.PlacementTool: Error while creating new item:', e
	    traceback.print_exc()
	    self.new_object = None
	    return 1

	if not self.new_object:
	    return 1

	if not self.new_object.parent:
	    self.new_object.set_property('parent', view.canvas.root)

	try:
	    self._move_item(view, event, self.new_object)
	except Exception, e:
	    print 'diacanvas.PlacementTool: Error while moving new item:', e
	    traceback.print_exc()
	    self.new_object.parent.remove(self.new_object)
	    self.new_object = None
	    return 1

	try:
	    self._grab_handle(view, event, self.new_object)
	except Exception, e:
	    print 'diacanvas.PlacementTool: Error while grabbing new item:', e
	    traceback.print_exc()
	    self.new_object.parent.remove(self.new_object)
	    self.new_object = None
	    return 1

	#view.canvas.set_property ('allow_undo', old_allow_undo)
	return 1

    def do_button_release_event(self, view, event):
	del self.new_object
	return 0

    def _create_item(self, view, event):
	"""
	Create a new object to be added to the canvas.
	Return: new canvas item.
	"""
	item = self.type()
	if self.properties and len(self.properties) > 0:
	    try:
		for (k,v) in self.properties.items():
		    item.set_property(k, v)
	    except TypeError, e:
	        print 'PlacementTool:', e
		traceback.print_exc()
	return item
	
    def _move_item(self, view, event, item):
	"""
	Move the newly created item to the desired position.
	"""
	wx, wy = view.window_to_world(event.x, event.y)
	ix, iy = item.affine_point_w2i(wx, wy)
	item.move(ix, iy)
	
    def _grab_handle(self, view, event, item):
	"""
	Grab a handle of the newly created canvas item. This will allow
	the user to instanty resize the newly created canvas item.
	"""
	if isinstance(item, dia.CanvasLine) and len(item.handles) > 0:
	    #print 'PlacementTool: setting handle of Line'
	    first = item.handles[0]
	    last = item.handles[-1]
	    wx, wy = view.window_to_world(event.x, event.y)
	    dist, glue, glue_to = view.canvas.glue_handle (first, wx, wy)
	    if glue_to and (dist <= view.handle_layer.glue_distance):
		glue_to.connect_handle(first)
	    view.handle_layer.grab_handle(last)
	elif isinstance(item, dia.CanvasElement):
	    #print 'PlacementTool: setting handle of Element'
	    handle = item.handles[dia.HANDLE_SE]
	    view.handle_layer.grab_handle(handle)
	else:
	    print 'diacanvas.PlacementTool: no handle handling for element', item

gobject.type_register(PlacementTool)
